Package: flowr
Type: Package
Title: Streamlining Design and Deployment of Complex Workflows
Description: This framework allows you to design and implement complex
    pipelines, and deploy them on your institution's computing cluster. This has
    been built keeping in mind the needs of bioinformatics workflows. However, it is
    easily extendable to any field where a series of steps (shell commands) are to
    be executed in a (work)flow.
Version: 0.9.10
Depends: R (>= 3.0.2), methods, params (>= 0.3), utils
Imports: diagram, whisker, tools
Suggests: reshape2, knitr, ggplot2, openxlsx, testthat, funr
VignetteBuilder: knitr
URL: https://github.com/sahilseth/flowr
BugReports: https://github.com/sahilseth/flowr/issues
Authors@R: c(person("Sahil", "Seth", role = c("aut", "cre"), email = "me@sahilseth.com"))
License: MIT + file LICENSE
RoxygenNote: 5.0.1
NeedsCompilation: no
Packaged: 2016-04-18 21:59:18 UTC; root
Author: Sahil Seth [aut, cre]
Maintainer: Sahil Seth <me@sahilseth.com>
Repository: CRAN
Date/Publication: 2016-04-19 01:17:29
Built: R 3.3.2; ; 2017-09-01 07:43:53 UTC; windows
