# Generated by roxygen2: do not edit by hand

S3method(check,flowdef)
S3method(check,flowmat)
S3method(get_status,character)
S3method(get_status,data.frame)
S3method(get_status,flow)
S3method(kill,character)
S3method(kill,flow)
S3method(plot,flow)
S3method(plot,flowdef)
S3method(plot_flow,character)
S3method(plot_flow,flow)
S3method(plot_flow,flowdef)
S3method(plot_flow,list)
S3method(rerun,character)
S3method(rerun,flow)
S3method(submit_flow,flow)
S3method(submit_flow,list)
S3method(to_flow,character)
S3method(to_flow,data.frame)
S3method(to_flow,flowmat)
S3method(to_flowdef,character)
S3method(to_flowdef,data.frame)
S3method(to_flowdef,flow)
S3method(to_flowdef,flowmat)
S3method(to_flowdef,tbl_df)
S3method(to_flowdet,character)
S3method(to_flowdet,flow)
S3method(to_flowdet,rootdir)
S3method(to_flowmat,data.frame)
S3method(to_flowmat,flow)
S3method(to_flowmat,list)
export(as.flowdef)
export(as.flowmat)
export(check)
export(check_args)
export(fetch)
export(fetch_conf)
export(fetch_pipes)
export(flow)
export(flowopts)
export(get_opts)
export(get_status)
export(get_unique_id)
export(get_wds)
export(is.flow)
export(is.flowdef)
export(is.flowmat)
export(job)
export(kill)
export(load_opts)
export(opts_flow)
export(plot_flow)
export(queue)
export(rerun)
export(run)
export(run_pipe)
export(set_opts)
export(setup)
export(status)
export(submit_flow)
export(submit_run)
export(test_queue)
export(to_flow)
export(to_flowdef)
export(to_flowdet)
export(to_flowmat)
export(whisker_render)
exportClasses(flow)
exportClasses(queue)
import(diagram)
import(methods)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.size)
importFrom(graphics,par)
importFrom(parallel,mclapply)
importFrom(params,kable)
importFrom(params,load_opts)
importFrom(params,new_opts)
importFrom(params,read_sheet)
importFrom(params,write_sheet)
importFrom(stats,complete.cases)
importFrom(stats,na.omit)
importFrom(tools,file_path_as_absolute)
importFrom(tools,file_path_sans_ext)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
importFrom(whisker,whisker.render)
